/*
 * lab1.c
 */

#include "lcd.h"
#include<stdio.h>

#include <avr/io.h>
#include <util/delay.h>


#define TASK_NR 0

/* current time, set by timer ISR */
volatile uint16_t current_time_ms_touch = 0u;

#define temp_max 50
#define temp_min 20


// TODO 2 hint
#define AREF 5

float temp_limit[] = {20, 25, 30, 35, 40, 45};
int temp_ant = 0;
int red_cycle, green_cycle, blue_cycle;

void adc_init()
{
    ADMUX = (1 << REFS0);
    ADCSRA = (1 << ADEN) | (5 << ADPS0);
}

uint16_t adc_read(uint8_t ch)
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | ch;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;

}

/*
Converts a value given by the ADC to volts.
*/
double ADC_voltage(int raw)
{
    (void)raw;
    return raw * AREF / 1024.;
}

void timer1_init()
{
    TCCR1A = (1 << COM1A0) | (0 << COM1A1);
    TCCR1A |= (1 << WGM10) | (0 << WGM11);
    
    TCCR1B = (1 << ICES1);
    TCCR1B |= (1 << WGM12) | (1 << WGM13);
    TCCR1B |= (1 << CS10);
}

void timer0_init()
{
    TCCR0A = (3 << WGM00) | (2 << COM0A0);
    TCCR0B = (1 << CS00);
    OCR0A = 0xC0;
}

void timer2_init()
{
    TCCR2A = (3 << WGM20) | (2 << COM2A0);
    TCCR2B = (1 << CS20);
    OCR2A = 0xC0;
}

void control(int temp)
{
    float out = (float)(temp - temp_min) / (temp_max - temp_min);
    if (temp < temp_min )
        out = 0;
    
    if (temp > temp_max)
       out = ICR1;
    
    if (temp >= temp_min && temp <= temp_max)
        out *= (float) ICR1;
    
    // albastru
    if (temp >= temp_limit[0] && temp < temp_limit[1]) {
        PORTB |= (1 << PB2);
        PORTB &= ~(1 << PB1);
        PORTB &= ~(1 << PB0);
    }

    // albastru + verde
    if (temp >= temp_limit[1] && temp < temp_limit[2]) {
        PORTB |= (1 << PB2);
        PORTB |= (1 << PB1);
        PORTB &= ~(1 << PB0);
    }

    // verde
    if (temp >= temp_limit[2] && temp < temp_limit[3]) {
        PORTB |= (1 << PB1);
        PORTB &= ~(1 << PB2);
        PORTB &= ~(1 << PB0);
    }

    // verde + rosu
    if (temp >= temp_limit[3] && temp < temp_limit[4]) {
        PORTB |= (1 << PB0);
        PORTB |= (1 << PB1);
        PORTB &= ~(1 << PB2);
    }

    // rosu
    if (temp >= temp_limit[4] && temp < temp_limit[5]) {
        PORTB |= (1 << PB0);
        PORTB &= ~(1 << PB2);
        PORTB &= ~(1 << PB1);
    }
    OCR1A = out;
}

int main()
{
    float voltage, tempVal;
    char buff[16];
    int tmpInt = 0, tmpFract = 0;
    float duty;
    const char *tmpSign;

    LCD_init();
	adc_init();
    timer1_init();
    timer0_init();
    timer2_init();

    TCNT1 = 0;
    ICR1 = 650;
    DDRD |= (1 << PD5);
    DDRB = 0xFF;
    
    while (1) {
        voltage = ADC_voltage(adc_read(3));
        float temperatureCelsius = (voltage - 0.5) * 100;

        tmpSign = (temperatureCelsius < 0) ? "-" : "";
        tempVal = (temperatureCelsius < 0) ? -temperatureCelsius : temperatureCelsius;
        tmpInt = (int) tempVal;
        tmpFract = (int) ((tempVal - tmpInt) * 1000);
        
        control(tmpInt);
        duty = (float) ((float)(OCR1A * 100 / ICR1));
        
        LCD_writeInstr(LCD_INSTR_clearDisplay);
		sprintf(buff, "Temp: %d.%dC", tmpInt, tmpFract);
        LCD_printAt(0x00, buff);

        sprintf(buff,"Duty: %d%", (int)duty);
        LCD_printAt(0x40, buff);
        _delay_ms(2000);
    }
    return 0;
}

